#include "lcd.h"
#include "usart.h"
#include "util.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

int counter = 0;
char *bauturi[4];

void finish(){
	counter = 0;
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	LCD_print("Choose a drink!");
	LCD_printAt(0x40,"OK");
	LCD_printAt(0x4C,"Next");
}

void app_print(int counter){

	LCD_writeInstr(LCD_INSTR_clearDisplay);

	for(int i=0; i<22; i++)
		USART0_print("\n\0");
	
	switch(counter){
		case 0:
			LCD_print(bauturi[counter]);
			LCD_printAt(0x40,"OK");
			LCD_printAt(0x4C,"Next");
			USART0_print("Alegeti bautura:\n->SUC DE MERE\n-suc de portocale\n-strong apple\n-drunk orage\0");
			break;
		case 1:
			LCD_print(bauturi[counter]);
			LCD_printAt(0x40,"OK\0");
			LCD_printAt(0x4C,"Next\0");
			USART0_print("Alegeti bautura:\n-suc de mere\n->SUC DE PORTOCALE\n-strong apple\n-drunk orage\0");
			break;
		case 2:
			LCD_print(bauturi[counter]);
			LCD_printAt(0x40,"OK\0");
			LCD_printAt(0x4C,"Next\0");	
			USART0_print("Alegeti bautura:\n-suc de mere\n-suc de portocale\n->STRONG APPLE\n-drunk orage\0");			
			break;
		case 3:
			LCD_print(bauturi[counter]);
			LCD_printAt(0x40,"OK\0");
			LCD_printAt(0x4C,"Next\0");
			USART0_print("Alegeti bautura:\n-suc de mere\n-suc de portocale\n-strong apple\n->DRUNK ORANGE\0");
			break;
		case 4:
			LCD_print("Se prepara...\0");
			LCD_printAt(0x40,"OK\0");
			LCD_printAt(0x4C,"Next\0");
			USART0_print("Bautura se prepara...\0");
			break;
		case 5:
			USART0_print("Your drink is ready. Enjoy! :)\0");
			LCD_print("Enjoy a drink!:)");
			LCD_printAt(0x40,"OK");
			LCD_printAt(0x4C,"Next");		
			break;
		default:
			break;
	}
}

void check_buttons()
{
	LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_print("Choose a drink!");
	LCD_printAt(0x40,"OK");
	LCD_printAt(0x4C,"Next");
	
	DDRD &= ~(1 << PD4);
	DDRD &= ~(1 << PD5);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD5);
	
	int8_t PD4_flag = 0, PD5_flag = 0;
	
    for(;;){
	
		if(!(PIND & (1 << PD4))  && !PD4_flag)
			{	
				PD4_flag = 1;
				app_print(counter);
				counter = (counter+1)%4;
			}
			
		if((PIND & (1 << PD4)) && PD4_flag)
			{
				PD4_flag = 0;
			}
		
		if (!(PIND & (1 << PD5))  && !PD5_flag)
			{
				//loading msg
				app_print(4);
				
						
				//activez pinul unei pompe in functie de bautura aleasa
				
				if (counter){
					DDRB |= (1<<PB3);
					PORTB |= (1<<PB3);
				}
		
				
				//folosesc LEDul pentru a arata ca bautura se prepara
				for(int i=0; i < 5; i++){
					_delay_ms(500);
					PORTD ^= (1<<PD7);
				}
				
				//finish msg
				app_print(5);
				
				//activez buzzerul pentru a anunta terminarea bauturii
				PORTD |= (1<<PD6);
				_delay_ms(2000);
				PORTD &= ~(1<<PD6);

				_delay_ms(2000);
				finish();
			}
		
		if((PIND & (1 << PD5)) && PD5_flag)
			{
				PD5_flag = 0;
			}
		
		_delay_ms(20);
	}
}


int main()
{
		
	DDRC |= (1<<PC2);
	PORTC |= (1<<PC2);
	
	//activez pd7 ca output pt LED
	DDRD |= (1 << PD7);
	PORTD |= (1<<PD7);

	//activez pd6 ca output pentru buzzer
	DDRD |= (1 << PD6);
	
	USART0_init();
    LCD_init();
	
	//initializam vectorul cu bauturi
	for(int i=0; i<4; i++)
		bauturi[i] = (char*)calloc(16, sizeof(char));
	
	strcpy(bauturi[0], "Suc de mere");
	strcpy(bauturi[1], "Suc de portocale");
	strcpy(bauturi[2], "Strong apple");
	strcpy(bauturi[3], "Drunk orange");
	
	//activez intreruperile
	sei();
	
	USART0_print("Alegeti bautura:\n->SUC DE MERE\n-suc de portocale\n-strong apple\n-drunk orage\0");
	
	check_buttons();
 
	//sterg intreruperile
	cli();
 
    return 0;
}

ISR(USART0_RX_vect)
{
	// Code to be executed when the USART receives a byte here
	char ReceivedByte;
	ReceivedByte = UDR0;
	 
	//cicleaza prin bauturi
	if( ReceivedByte == 1){

		app_print(counter);
		counter++;
		if( counter == 4)
			counter = 0;
	}
	
	//alege optiunea dorita
	if (ReceivedByte == 0){
		
		//loading msg
		app_print(4);		
		
		//activez pinul unei pompe in functie de bautura aleasa
		
		if (counter){
			DDRB |= (1<<PB3);
			PORTB |= (1<<PB3);
		}
		
		
		//LEDul este aprins intermitent cat timp se preapara bautura
		for(int i=0; i < 10; i++){
			_delay_ms(500);
			PORTD ^= (1<<PD7);
		}
		
		
		PORTB &= ~(1<<PB3);
		//finish msg
		app_print(5);
		PORTD |= (1<<PD7);
		
		//pornesc buzzerul pentru a anunta terminarea prepararii
		PORTD |= (1<<PD6);
		_delay_ms(2000);
		PORTD &= ~(1<<PD6);

		_delay_ms(2000);
		finish();
	}
}
